from ..schemas import QueryBase
from typing import Optional, Literal


def cleanse_query_for_search(query: Optional[QueryBase], offset_type: Literal["int", "str"] = "int") -> dict:
    if offset_type == "int":
        assert query.offset is None or isinstance(query.offset, int), "Offset must be int in Qdrant searching"
    elif offset_type == "str":
        assert query.offset is None or isinstance(query.offset, str), "Offset must be id (str) in Qdrant querying"
    assert query.id is None, "QueryBase's 'id' key should not being use for similarity search in qdrant"
    assert query.ids is None, "QueryBase's 'ids' key should not being use for similarity search in qdrant"
    assert query.order_by is None, "QueryBase's 'order_by' key should not being use for similarity search in qdrant"
    assert query.filters is None or isinstance(query.filters, dict), "Query.filters only accept dict of qdrant query"

    if query is None: return {}
    base = query.filters if query.filters is not None else {}

    # forbidden id and ids in filters
    if "id" in base: raise AssertionError(
        f"You should not include 'id' in your QueryBase.filters when searching Qdrant")
    if "ids" in base: raise AssertionError(
        f"You should not include 'ids' in your QueryBase.filters when searching Qdrant")
    return base



from ..session import Db
from redis import Redis
from pymongo import MongoClient
from sqlmodel import Session
from qdrant_client import QdrantClient


class ApiDbDeps:

    def __init__(self, db: Db):
        self.__db = db

    def get_redis(self) -> Redis:
        r = None
        try:
            r = self.__db.get_redis()
            yield r
        finally:
            self.__db.try_close(r)

    def get_mongo(self) -> MongoClient:
        m = None
        try:
            m = self.__db.get_mongo()
            yield m
        finally:
            self.__db.try_close(m)

    def get_write_sql(self) -> Session:
        s = None
        try:
            s = self.__db.get_sql_write_client()
            yield s
        finally:
            self.__db.try_close(s)

    def get_qdrant(self) -> QdrantClient:
        q = None
        try:
            q = self.__db.get_qdrant_client()
            yield q
        finally:
            self.__db.try_close(q)
